# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/05_sig.ipynb.

# %% auto 0
__all__ = ['getposdef', 'getkwodef', 'getspecdef', 'getsigdef', 'getspeckws', 'getsigkws']

# %% ../nbs/05_sig.ipynb 4
from typing import Callable
from .types import (ArgsTuple,)
from .utils import (filterargvals, safegetspec, safegetsig)

# %% ../nbs/05_sig.ipynb 7
def getposdef(func: Callable) -> ArgsTuple:    
    '''
    Retrieve all positional arguments with their default values from a function.

    Parameters
    ----------
    func : Callable
        The function to inspect.

    Returns
    -------
    Tuple[Tuple[str, Any], ...]
        Tuple of positional arguments and their default values.

    Notes
    -----
    Uses `inspect.getfullargspec` to retrieve the arguments.
    '''
    # spec = inspect.getfullargspec(func)
    spec = safegetspec(func)
    if spec.defaults is None: return tuple()
    
    # NOTE: for readability, we assume that the last n arguments are the defaults
    nvals = len(spec.defaults)
    names = spec.args[-nvals:]
    adict = dict(zip(names, spec.defaults))
    return filterargvals(**adict)

def getkwodef(func: Callable) -> ArgsTuple:    
    '''
    Retrieve all keyword-only arguments with their default values from a function.

    Parameters
    ----------
    func : Callable
        The function to inspect.

    Returns
    -------
    Tuple[Tuple[str, Any], ...]
        Tuple of keyword-only arguments and their default values.

    Notes
    -----
    Uses `inspect.kwonlyargs` to retrieve the arguments.
    '''
    # spec = inspect.getfullargspec(func)
    spec = safegetspec(func)
    if spec.kwonlydefaults is None: return tuple()
    return filterargvals(**spec.kwonlydefaults)


# %% ../nbs/05_sig.ipynb 9
def getspecdef(func: Callable) -> ArgsTuple:
    '''
    Retrieve both positional and keyword-only arguments with their default values from a function.

    Parameters
    ----------
    func : Callable
        The function to inspect.

    Returns
    -------
    Tuple[Tuple[str, Any], ...]
        Tuple of keyword-only arguments and their default values.

    Notes
    -----
    Uses `inspect.kwonlyargs` to retrieve the arguments.
    '''
    return getposdef(func) + getkwodef(func)


# %% ../nbs/05_sig.ipynb 11
def getsigdef(func: Callable) -> ArgsTuple:
    '''
    Retrieve all parameters with their default values from a function.

    Parameters
    ----------
    func : Callable
        The function to inspect.

    Returns
    -------
    Tuple[Tuple[str, Any], ...]
        Tuple of keyword-only arguments and their default values.

    Notes
    -----
    Uses `inspect.signature` to retrieve the parameters.
    '''    
    spec = safegetsig(func)
    args = {p.name: p.default for p in spec.parameters.values()}
    args = filterargvals(**args)
    return args

# %% ../nbs/05_sig.ipynb 13
def getspeckws(obj) -> dict:
    '''Get signature keywords with defaults only'''
    return dict(getspecdef(obj)) # arguments with defaults

def getsigkws(obj) -> dict:    
    '''Get all parameters with defaults.'''
    return dict(getsigdef(obj)) # arguments with defaults
